@@ -9,6 +9,7 @@
 #include <stdbool.h>
 
 /* library includes */
+#include <uv/errno.h>
 #include <uv.h>
 #include <systemd/sd-daemon.h>
 
@@ -166,7 +167,11 @@ static void dump_addrinfo(FILE *f, const char *label, const struct addrinfo *a,
   if (ia) {
     if (inet_ntop(a->ai_family, ia, paddr, sizeof(paddr)) == NULL) {
       int e = errno;
-      strerror_r(e, paddr+6, sizeof(paddr)-(6+1));
+      /* these shenanigans are all for the GNU variant of
+         strerror_r(3), not the XSI variant: */
+      const char * e2 = strerror_r(e, paddr+6, sizeof(paddr)-(6+1));
+      if (e2 != paddr+6)
+        strncpy(paddr+6, e2, sizeof(paddr)-(6+1));
       memcpy(paddr, "<err> ", 6);
       paddr[sizeof(paddr)-1] = '\0';
     }
@@ -500,7 +505,7 @@ void inbound_first_read(uv_stream_t* stream, ssize_t nread, const uv_buf_t* buf)
   assert((unsigned char*)(buf->base) == st->x[0].staging + st->x[0].bytes_read);
   struct demuxer *demuxer = stream->loop->data;
 
-  if (nread == 0 || nread == UV__EAGAIN || nread == UV__EBUSY) {
+  if (nread == 0 || nread == UV_EAGAIN || nread == UV_EBUSY) {
     fprintf(stderr, "[%d] ignoring weird inbound_first_read() with status: (%zd), %s\n",
             st->id, nread, uv_strerror(nread));
     return;
@@ -653,14 +658,14 @@ void stream_read(uv_stream_t* stream, ssize_t nread, const uv_buf_t* buf) {
       assert(!st->x[ix].eof_seen);
       touch_streamstate(st);
       
-      if (nread == 0 || nread == UV__EAGAIN || nread == UV__EBUSY ||
-          nread ==UV__ENOBUFS || nread == UV_EINTR) {
+      if (nread == 0 || nread == UV_EAGAIN || nread == UV_EBUSY ||
+          nread ==UV_ENOBUFS || nread == UV_EINTR) {
         fprintf(stderr, "[%d] ignoring weird stream_read() with status: (%zd), %s\n",
                 st->id, nread, uv_strerror(nread));
         return;
       }
   
-      if (nread == UV__EOF) {
+      if (nread == UV_EOF) {
         if (demuxer->debug)
           fprintf(stderr, "[%d] EOF received from %s leg\n", st->id, indir);
         /* we've just seen the EOF from one channel -- shut down the
@@ -872,7 +877,7 @@ int main(int argc, const char **argv) {
   freeaddrinfo(demuxer->http_targets);
   freeaddrinfo(demuxer->dns_targets);
   free_demuxer(demuxer);
-  if ((err == uv_loop_close(&loop)))
+  if ((err = uv_loop_close(&loop)))
     fprintf(stderr, "uv_loop_close() failed: (%d) %s\n", err, uv_strerror(err));
   
   return 0;
